CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
451 lines
17 KiB
C++
451 lines
17 KiB
C++
/* ************************************************************************
|
|
* Copyright (C) 2016-2025 Advanced Micro Devices, Inc. All rights reserved.
|
|
*
|
|
* Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
* of this software and associated documentation files (the "Software"), to deal
|
|
* in the Software without restriction, including without limitation the rights
|
|
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
* copies of the Software, and to permit persons to whom the Software is
|
|
* furnished to do so, subject to the following conditions:
|
|
*
|
|
* The above copyright notice and this permission notice shall be included in
|
|
* all copies or substantial portions of the Software.
|
|
*
|
|
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
|
|
* SOFTWARE.
|
|
*
|
|
* ************************************************************************ */
|
|
|
|
#include <fstream>
|
|
#include <iostream>
|
|
#include <stdlib.h>
|
|
#include <vector>
|
|
|
|
#include "testing_common.hpp"
|
|
|
|
/* ============================================================================================ */
|
|
|
|
using hipblasHpmvBatchedModel
|
|
= ArgumentModel<e_a_type, e_uplo, e_N, e_alpha, e_incx, e_beta, e_incy, e_batch_count>;
|
|
|
|
inline void testname_hpmv_batched(const Arguments& arg, std::string& name)
|
|
{
|
|
hipblasHpmvBatchedModel{}.test_name(arg, name);
|
|
}
|
|
|
|
template <typename T>
|
|
void testing_hpmv_batched_bad_arg(const Arguments& arg)
|
|
{
|
|
using Ts = hipblas_internal_type<T>;
|
|
bool FORTRAN = arg.api == hipblas_client_api::FORTRAN;
|
|
auto hipblasHpmvBatchedFn
|
|
= FORTRAN ? hipblasHpmvBatched<T, true> : hipblasHpmvBatched<T, false>;
|
|
|
|
auto hipblasHpmvBatchedFn_64
|
|
= arg.api == FORTRAN_64 ? hipblasHpmvBatched_64<T, true> : hipblasHpmvBatched_64<T, false>;
|
|
|
|
for(auto pointer_mode : {HIPBLAS_POINTER_MODE_HOST, HIPBLAS_POINTER_MODE_DEVICE})
|
|
{
|
|
hipblasLocalHandle handle(arg);
|
|
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, pointer_mode));
|
|
|
|
hipblasFillMode_t uplo = HIPBLAS_FILL_MODE_UPPER;
|
|
int64_t N = 100;
|
|
int64_t incx = 1;
|
|
int64_t incy = 1;
|
|
int64_t batch_count = 2;
|
|
int64_t A_size = N * (N + 1) / 2;
|
|
|
|
device_vector<T> d_alpha(1), d_beta(1), d_one(1), d_zero(1);
|
|
|
|
Ts h_alpha{1}, h_beta{2}, h_one{1}, h_zero{0};
|
|
if constexpr(is_complex<T>)
|
|
h_one = {1, 0};
|
|
|
|
const Ts* alpha = &h_alpha;
|
|
const Ts* beta = &h_beta;
|
|
const Ts* one = &h_one;
|
|
const Ts* zero = &h_zero;
|
|
|
|
if(pointer_mode == HIPBLAS_POINTER_MODE_DEVICE)
|
|
{
|
|
CHECK_HIP_ERROR(hipMemcpy(d_alpha, alpha, sizeof(*alpha), hipMemcpyHostToDevice));
|
|
CHECK_HIP_ERROR(hipMemcpy(d_beta, beta, sizeof(*beta), hipMemcpyHostToDevice));
|
|
CHECK_HIP_ERROR(hipMemcpy(d_one, one, sizeof(*one), hipMemcpyHostToDevice));
|
|
CHECK_HIP_ERROR(hipMemcpy(d_zero, zero, sizeof(*zero), hipMemcpyHostToDevice));
|
|
alpha = d_alpha;
|
|
beta = d_beta;
|
|
one = d_one;
|
|
zero = d_zero;
|
|
}
|
|
|
|
device_batch_vector<T> dAp(1, hipblas_packed_matrix_size(N), 1, batch_count);
|
|
device_batch_vector<T> dx(N, incx, batch_count);
|
|
device_batch_vector<T> dy(N, incy, batch_count);
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_NOT_INITIALIZED,
|
|
hipblasHpmvBatchedFn,
|
|
(nullptr,
|
|
uplo,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
HIPBLAS_FILL_MODE_FULL,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_ENUM,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
(hipblasFillMode_t)HIPBLAS_OP_N,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
nullptr,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
nullptr,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
if(pointer_mode == HIPBLAS_POINTER_MODE_HOST)
|
|
{
|
|
// For device mode in rocBLAS we don't have checks for dAp, dx, dy as we may be able to quick return
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
alpha,
|
|
nullptr,
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
nullptr,
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
DAPI_EXPECT(HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
beta,
|
|
nullptr,
|
|
incy,
|
|
batch_count));
|
|
|
|
// rocBLAS implementation has alpha == 0 quick return after arg checks, so if we're using 32-bit params,
|
|
// this should fail with invalid-value as c_i32_overflow will rollover to -2147483648
|
|
// Note: that this strategy can't check incx as rocBLAS supports negative. Also depends on implementation so not testing cuBLAS for now
|
|
DAPI_EXPECT((arg.api & c_API_64) ? HIPBLAS_STATUS_SUCCESS
|
|
: HIPBLAS_STATUS_INVALID_VALUE,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
c_i32_overflow,
|
|
zero,
|
|
nullptr,
|
|
nullptr,
|
|
incx,
|
|
one,
|
|
nullptr,
|
|
incy,
|
|
batch_count));
|
|
}
|
|
|
|
// With N == 0, can have all nullptrs
|
|
DAPI_CHECK(hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
0,
|
|
nullptr,
|
|
nullptr,
|
|
nullptr,
|
|
incx,
|
|
nullptr,
|
|
nullptr,
|
|
incy,
|
|
batch_count));
|
|
DAPI_CHECK(hipblasHpmvBatchedFn,
|
|
(handle, uplo, N, nullptr, nullptr, nullptr, incx, nullptr, nullptr, incy, 0));
|
|
|
|
// With alpha == 0 can have A and x nullptr
|
|
DAPI_CHECK(hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
zero,
|
|
nullptr,
|
|
nullptr,
|
|
incx,
|
|
beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
// With alpha == 0 && beta == 1, all other ptrs can be nullptr
|
|
DAPI_CHECK(
|
|
hipblasHpmvBatchedFn,
|
|
(handle, uplo, N, zero, nullptr, nullptr, incx, one, nullptr, incy, batch_count));
|
|
}
|
|
}
|
|
|
|
template <typename T>
|
|
void testing_hpmv_batched(const Arguments& arg)
|
|
{
|
|
using Ts = hipblas_internal_type<T>;
|
|
bool FORTRAN = arg.api == hipblas_client_api::FORTRAN;
|
|
auto hipblasHpmvBatchedFn
|
|
= FORTRAN ? hipblasHpmvBatched<T, true> : hipblasHpmvBatched<T, false>;
|
|
|
|
auto hipblasHpmvBatchedFn_64
|
|
= arg.api == FORTRAN_64 ? hipblasHpmvBatched_64<T, true> : hipblasHpmvBatched_64<T, false>;
|
|
|
|
hipblasFillMode_t uplo = char2hipblas_fill(arg.uplo);
|
|
int64_t N = arg.N;
|
|
int64_t incx = arg.incx;
|
|
int64_t incy = arg.incy;
|
|
int64_t batch_count = arg.batch_count;
|
|
|
|
size_t abs_incy = incy >= 0 ? incy : -incy;
|
|
size_t A_size = N * (N + 1) / 2;
|
|
|
|
hipblasLocalHandle handle(arg);
|
|
|
|
// argument sanity check, quick return if input parameters are invalid before allocating invalid
|
|
// memory
|
|
bool invalid_size = N < 0 || !incx || !incy || batch_count < 0;
|
|
if(invalid_size || !N || !batch_count)
|
|
{
|
|
DAPI_EXPECT(invalid_size ? HIPBLAS_STATUS_INVALID_VALUE : HIPBLAS_STATUS_SUCCESS,
|
|
hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
nullptr,
|
|
nullptr,
|
|
nullptr,
|
|
incx,
|
|
nullptr,
|
|
nullptr,
|
|
incy,
|
|
batch_count));
|
|
return;
|
|
}
|
|
|
|
double hipblas_error_host{0}, hipblas_error_device{0};
|
|
|
|
T h_alpha = arg.get_alpha<T>();
|
|
T h_beta = arg.get_beta<T>();
|
|
|
|
// arrays of pointers-to-host on host
|
|
host_batch_matrix<T> hA(N, N, N, batch_count);
|
|
host_batch_matrix<T> hAp(1, hipblas_packed_matrix_size(N), 1, batch_count);
|
|
host_batch_vector<T> hx(N, incx, batch_count);
|
|
host_batch_vector<T> hy(N, incy, batch_count);
|
|
host_batch_vector<T> hy_cpu(N, incy, batch_count);
|
|
host_batch_vector<T> hy_host(N, incy, batch_count);
|
|
host_batch_vector<T> hy_device(N, incy, batch_count);
|
|
|
|
// arrays of pointers-to-device on host
|
|
device_batch_matrix<T> dAp(1, hipblas_packed_matrix_size(N), 1, batch_count);
|
|
device_batch_vector<T> dx(N, incx, batch_count);
|
|
device_batch_vector<T> dy(N, incy, batch_count);
|
|
device_vector<T> d_alpha(1);
|
|
device_vector<T> d_beta(1);
|
|
|
|
// Check device memory allocation
|
|
CHECK_DEVICE_ALLOCATION(dAp.memcheck());
|
|
CHECK_DEVICE_ALLOCATION(dx.memcheck());
|
|
CHECK_DEVICE_ALLOCATION(dy.memcheck());
|
|
CHECK_DEVICE_ALLOCATION(d_alpha.memcheck());
|
|
CHECK_DEVICE_ALLOCATION(d_beta.memcheck());
|
|
|
|
// Initial Data on CPU
|
|
hipblas_init_matrix(hA, arg, hipblas_client_never_set_nan, hipblas_hermitian_matrix, true);
|
|
hipblas_init_vector(hx, arg, hipblas_client_alpha_sets_nan, false, true);
|
|
hipblas_init_vector(hy, arg, hipblas_client_alpha_sets_nan);
|
|
|
|
// helper function to convert Regular matrix `hA` to packed matrix `hAp`
|
|
regular_to_packed(uplo == HIPBLAS_FILL_MODE_UPPER, hA, hAp, N);
|
|
|
|
// copy vector
|
|
hy_cpu.copy_from(hy);
|
|
|
|
// copy data from CPU to device
|
|
CHECK_HIP_ERROR(dAp.transfer_from(hAp));
|
|
CHECK_HIP_ERROR(dx.transfer_from(hx));
|
|
CHECK_HIP_ERROR(dy.transfer_from(hy));
|
|
CHECK_HIP_ERROR(hipMemcpy(d_alpha, &h_alpha, sizeof(T), hipMemcpyHostToDevice));
|
|
CHECK_HIP_ERROR(hipMemcpy(d_beta, &h_beta, sizeof(T), hipMemcpyHostToDevice));
|
|
|
|
if(arg.unit_check || arg.norm_check)
|
|
{
|
|
/* =====================================================================
|
|
HIPBLAS
|
|
=================================================================== */
|
|
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_HOST));
|
|
DAPI_CHECK(hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
reinterpret_cast<Ts*>(&h_alpha),
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
reinterpret_cast<Ts*>(&h_beta),
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
CHECK_HIP_ERROR(hy_host.transfer_from(dy));
|
|
CHECK_HIP_ERROR(dy.transfer_from(hy));
|
|
|
|
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_DEVICE));
|
|
DAPI_CHECK(hipblasHpmvBatchedFn,
|
|
(handle,
|
|
uplo,
|
|
N,
|
|
d_alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
d_beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
|
|
CHECK_HIP_ERROR(hy_device.transfer_from(dy));
|
|
|
|
/* =====================================================================
|
|
CPU BLAS
|
|
=================================================================== */
|
|
|
|
for(size_t b = 0; b < batch_count; b++)
|
|
{
|
|
ref_hpmv<T>(uplo, N, h_alpha, hAp[b], hx[b], incx, h_beta, hy_cpu[b], incy);
|
|
}
|
|
|
|
// enable unit check, notice unit check is not invasive, but norm check is,
|
|
// unit check and norm check can not be interchanged their order
|
|
if(arg.unit_check)
|
|
{
|
|
unit_check_general<T>(1, N, batch_count, abs_incy, hy_cpu, hy_host);
|
|
unit_check_general<T>(1, N, batch_count, abs_incy, hy_cpu, hy_device);
|
|
}
|
|
if(arg.norm_check)
|
|
{
|
|
hipblas_error_host
|
|
= norm_check_general<T>('F', 1, N, abs_incy, hy_cpu, hy_host, batch_count);
|
|
hipblas_error_device
|
|
= norm_check_general<T>('F', 1, N, abs_incy, hy_cpu, hy_device, batch_count);
|
|
}
|
|
}
|
|
|
|
if(arg.timing)
|
|
{
|
|
double gpu_time_used{0};
|
|
CHECK_HIP_ERROR(dy.transfer_from(hy));
|
|
hipStream_t stream;
|
|
CHECK_HIPBLAS_ERROR(hipblasGetStream(handle, &stream));
|
|
CHECK_HIPBLAS_ERROR(hipblasSetPointerMode(handle, HIPBLAS_POINTER_MODE_DEVICE));
|
|
|
|
int runs = arg.cold_iters + arg.iters;
|
|
for(int iter = 0; iter < runs; iter++)
|
|
{
|
|
if(iter == arg.cold_iters)
|
|
gpu_time_used = get_time_us_sync(stream);
|
|
|
|
CHECK_HIPBLAS_ERROR(hipblasHpmvBatchedFn(handle,
|
|
uplo,
|
|
N,
|
|
d_alpha,
|
|
dAp.ptr_on_device(),
|
|
dx.ptr_on_device(),
|
|
incx,
|
|
d_beta,
|
|
dy.ptr_on_device(),
|
|
incy,
|
|
batch_count));
|
|
}
|
|
gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
|
|
|
|
hipblasHpmvBatchedModel{}.log_args<T>(std::cout,
|
|
arg,
|
|
gpu_time_used,
|
|
hpmv_gflop_count<T>(N),
|
|
hpmv_gbyte_count<T>(N),
|
|
hipblas_error_host,
|
|
hipblas_error_device);
|
|
}
|
|
}
|